@@ -1,15 +1,11 @@
 ## Import and configuration
 import json
-import re
-from googleapiclient.errors import HttpError
-from datetime import datetime, timedelta, date
-import pandas as pd
-from configparser import ConfigParser
-import sys
 import os
-import xlrd
-import math
-import pytz
+from configparser import ConfigParser
+import pandas as pd
+from googleapiclient.errors import HttpError
+import asyncio
+
 from clarku_youtube_crawler.CrawlerObject import _CrawlerObject
 
 CONFIG = "config.ini"
@@ -24,22 +20,24 @@ class ChannelCrawler(_CrawlerObject):
     def search_channel(self, search_key):
         self.search_key = search_key
         search_key_dir = f"{self.CURRENT_ROOT}{self.channel_search_files}{self.search_key}.json"
-        if os.path.exists(search_key_dir):
-            os.remove(search_key_dir)
-        self._crawl_channels(search_key_dir)
+        if not os.path.exists(search_key_dir):
+            self._crawl_channels(search_key_dir)
+        else:
+            print(f'{search_key_dir} existed. Skip search key {search_key}')
 
     def _crawl_channels(self, file_path):
         response = self._search_channel(file_path=file_path)
-        total_result = response["pageInfo"]["totalResults"]
-        if "nextPageToken" not in response:
-            print(f"total results:{str(total_result)}")
-            return
-        while True:
-            response = self._search_channel(file_path=file_path,
-                                            page_token=response["nextPageToken"])
+        if response != "error":
+            total_result = response["pageInfo"]["totalResults"]
             if "nextPageToken" not in response:
                 print(f"total results:{str(total_result)}")
-                break
+                return
+            while True:
+                response = self._search_channel(file_path=file_path,
+                                                page_token=response["nextPageToken"])
+                if "nextPageToken" not in response:
+                    print(f"total results:{str(total_result)}")
+                    break
 
     def _search_channel(self, file_path, page_token=None):
         """
@@ -69,9 +67,8 @@ class ChannelCrawler(_CrawlerObject):
             self._write_item(file_path, response["items"])  # remove duplicate
             return response
         except HttpError as e:
-            error = json.loads(e.content)["error"]["errors"][0]["reason"]
-            print(error)
-            if error == "dailyLimitExceeded" or error == "quotaExceeded":
+            error = self._get_error_code(e.content)
+            if error == "update_API_key":
                 self._try_next_id()
                 return self._search_channel(file_path, page_token)
         except Exception as e:
@@ -112,6 +109,7 @@ class ChannelCrawler(_CrawlerObject):
 
     def crawl(self, **kwargs):
         """
+        async version to crawl channels
         crawl all videos belong to a list of channels (specified by channelIds)
         :param kwargs: specify channel_header to configure which column contains channel id
         :return:
@@ -119,6 +117,7 @@ class ChannelCrawler(_CrawlerObject):
         filename = kwargs.get("filename", f"{self.CURRENT_ROOT}channels_to_collect.csv")
         header = kwargs.get("channel_header", "channelId")
         search_key_subdir = kwargs.get("save_to", f"{self.CURRENT_ROOT}{self.video_search_files}")
+        core = kwargs.get("core", 5)
         accepted_ext = [".csv", ".xlsx"]
 
         file, ext = os.path.splitext(filename)
@@ -129,8 +128,8 @@ class ChannelCrawler(_CrawlerObject):
             df = pd.read_csv(filename)
         elif ext == ".xlsx":
             df = pd.read_excel(filename)
-        self.channel_list = set(df[header])
-
+        self.channel_list = list(set(df[header]))
+        # asyncio.run(self.start_crawling(core, search_key_subdir))
         try:
             os.mkdir(search_key_subdir)
         except OSError:
@@ -138,15 +137,26 @@ class ChannelCrawler(_CrawlerObject):
         else:
             print("Successfully created the directory %s " % search_key_subdir)
 
-        for channelId in self.channel_list:
+        asyncio.run(self.start_crawling(core, search_key_subdir))
+
+    async def start_crawling(self, core, search_key_subdir):
+        for idx in range(0, len(self.channel_list), core):
+            sublist = self.channel_list[idx:idx + core]
+            await self.crawl_sublist_async(sublist, search_key_subdir)
+
+    async def crawl_sublist_async(self, sublist, search_key_subdir):
+        for channelId in sublist:
             dir = f"{search_key_subdir}/{channelId}.json"
             if not self.isCrawled(dir):
                 print(f"Crawling a video list from {channelId}....")
-                self._search_videos_from_channels(dir, channelId)
+                if channelId != sublist[-1]:  # last element needs "await"
+                    asyncio.gather(self._search_videos_from_channels_async(dir, channelId))
+                else:
+                    await asyncio.gather(self._search_videos_from_channels_async(dir, channelId))
             else:
                 print(f"Skip {channelId}, channel already crawled")
 
-    def _search_videos_from_channels(self, file_name, channel_id):
+    async def _search_videos_from_channels_async(self, file_name, channel_id):
         response = self._search_all_videos_in_channel(file_name, channel_id)
         total_results = response["pageInfo"]["totalResults"]
         print(f"Total videos: {total_results}")
@@ -181,11 +191,11 @@ class ChannelCrawler(_CrawlerObject):
             self._write_item(file_path, response["items"])
             return response
         except HttpError as e:
-            error = json.loads(e.content)["error"]["errors"][0]["reason"]
-            if error == "dailyLimitExceeded" or error == "quotaExceeded":
+            error = self._get_error_code(e.content)
+            if error == "update_API_key":
                 self._try_next_id()
                 return self._search_all_videos_in_channel(file_path, channel_id,
                                                           page_token)  # I assume it's missing one var
         except Exception as e:
             print(e)
-            return "error"
\ No newline at end of file
+            return "error"
