@@ -129,11 +129,12 @@ class _CrawlerObject():
                 return "error"
             return response["items"][0]
         except HttpError as e:
-            error = json.loads(e.content)["error"]["errors"][0]["reason"]
-            if error == "dailyLimitExceeded" or error == "quotaExceeded":
+            error = self._get_error_code(e.content)
+            if error == "update_API_key":
                 self._try_next_id()
                 return self.get_video(video_id)
         except Exception as e:
+            print(e)
             return "error"
 
     def get_comments(self, video_id, comment_page_count):
@@ -163,8 +164,8 @@ class _CrawlerObject():
                 counter += 1
             return comments
         except HttpError as e:
-            error = json.loads(e.content)["error"]["errors"][0]["reason"]
-            if error == "dailyLimitExceeded" or error == "quotaExceeded":
+            error = self._get_error_code(e.content)
+            if error == "update_API_key":
                 self._try_next_id()
                 return self.get_comments(video_id, comment_page_count)
         except Exception as e:
@@ -182,8 +183,8 @@ class _CrawlerObject():
             response = self.youtube.channels().list(part=part, maxResults=1, id=channel_id).execute()
             return response["items"][0]
         except HttpError as e:
-            error = json.loads(e.content)["error"]["errors"][0]["reason"]
-            if error == "dailyLimitExceeded" or error == "quotaExceeded":
+            error = self._get_error_code(e.content)
+            if error == "update_API_key":
                 self._try_next_id()
                 return self.get_channel(channel_id)
         except Exception as e:
@@ -220,3 +221,15 @@ class _CrawlerObject():
         with open(file_path, 'a+') as fp:
             for item in items:
                 fp.write(json.dumps(item) + "\n")
+
+    def _get_error_code(self, message):
+        error = json.loads(message)
+        reason = "unknown"
+        if error["error"]["errors"][0]["reason"]:
+            reason = error["error"]["errors"][0]["reason"]
+        if reason == "dailyLimitExceeded" or reason == "quotaExceeded":
+            print("Running out of quotas. Updating API key.")
+            return "update_API_key"
+        elif reason == 'badRequest':
+            print("API deprecated. Delete API from key file.")
+            return "update_API_key"
